@@ -10,7 +10,7 @@
     :license: BSD, see LICENSE for more details.
 """
 
-nodeenv_version = '0.10.0'
+nodeenv_version = '0.11.0'
 
 import sys
 import os
@@ -22,18 +22,22 @@ import pipes
 
 try:
     from ConfigParser import SafeConfigParser as ConfigParser
+    iteritems = lambda dict_: dict_.iteritems()
 except ImportError:
     # Python 3
     from configparser import ConfigParser
+    iteritems = lambda dict_: dict_.items()
+
 
 from pkg_resources import parse_version
-from contextlib import closing
 
 join = os.path.join
 abspath = os.path.abspath
 
 # ---------------------------------------------------------
 # Utils
+
+
 class Config(object):
     """
     Configuration namespace.
@@ -51,18 +55,25 @@ class Config(object):
     prebuilt = False
 
     @classmethod
-    def _load(cls, configfile=None):
+    def _load(cls, configfiles, verbose=False):
         """
-        Load configuration from given file or "~/.nodeenvrc".
+        Load configuration from the given files in reverse order,
+        if they exist and have a [nodeenv] section.
         """
-        configfile = configfile or os.path.expanduser("~/.nodeenvrc")
-        if os.path.exists(configfile):
+        for configfile in reversed(configfiles):
+            configfile = os.path.expanduser(configfile)
+            if not os.path.exists(configfile):
+                continue
+
             ini_file = ConfigParser()
             ini_file.read(configfile)
-
             section = "nodeenv"
-            for attr, val in vars(cls).iteritems():
-                if attr.startswith('_') or not ini_file.has_option(section, attr):
+            if not ini_file.has_section(section):
+                continue
+
+            for attr, val in iteritems(vars(cls)):
+                if attr.startswith('_') or not \
+                   ini_file.has_option(section, attr):
                     continue
 
                 if isinstance(val, bool):
@@ -70,6 +81,9 @@ class Config(object):
                 else:
                     val = ini_file.get(section, attr)
 
+                if verbose:
+                    print('CONFIG {0}: {1} = {2}'.format(
+                        os.path.basename(configfile), attr, val))
                 setattr(cls, attr, val)
 
     @classmethod
@@ -78,11 +92,17 @@ class Config(object):
         Print defaults for the README.
         """
         print ("    [nodeenv]")
-        print ("    " + "\n    ".join("%s = %s" % (k, v)
-            for k, v in sorted(vars(cls).iteritems())
+        print ("    " + "\n    ".join(
+            "%s = %s" % (k, v) for k, v in sorted(iteritems(vars(cls)))
             if not k.startswith('_')))
 
 
+Config._default = dict(
+    (attr, val) for attr, val in iteritems(vars(Config))
+    if not attr.startswith('_')
+)
+
+
 def clear_output(out):
     """
     Remove new-lines and
@@ -141,9 +161,11 @@ def create_logger():
 logger = create_logger()
 
 
-def parse_args():
+def parse_args(check=True):
     """
-    Parses command line arguments
+    Parses command line arguments.
+
+    Set `check` to False to skip validation checks.
     """
     parser = optparse.OptionParser(
         version=nodeenv_version,
@@ -153,7 +175,8 @@ def parse_args():
         '-n', '--node', dest='node', metavar='NODE_VER', default=Config.node,
         help='The node.js version to use, e.g., '
         '--node=0.4.3 will use the node-v0.4.3 '
-        'to create the new environment. The default is last stable version. '
+        'to create the new environment. '
+        'The default is last stable version (`latest`). '
         'Use `system` to use system-wide node.')
 
     parser.add_option(
@@ -174,7 +197,12 @@ def parse_args():
     parser.add_option(
         '-q', '--quiet',
         action='store_true', dest='quiet', default=False,
-        help="Quete mode")
+        help="Quiet mode")
+
+    parser.add_option(
+        '-C', '--config-file', dest='config_file', default=None,
+        help="Load a different file than '~/.nodeenvrc'. "
+        "Pass an empty string for no config (use built-in defaults).")
 
     parser.add_option(
         '-r', '--requirements',
@@ -222,7 +250,8 @@ def parse_args():
         metavar='NPM_VER', default=Config.npm,
         help='The npm version to use, e.g., '
         '--npm=0.3.18 will use the npm-0.3.18.tgz '
-        'tarball to install. The default is last available version.')
+        'tarball to install. '
+        'The default is last available version (`latest`).')
 
     parser.add_option(
         '--no-npm-clean', dest='no_npm_clean',
@@ -256,19 +285,28 @@ def parse_args():
         help='Install node.js from prebuilt package')
 
     options, args = parser.parse_args()
+    if options.config_file is None:
+        options.config_file = ["./setup.cfg", "~/.nodeenvrc"]
+    elif not options.config_file:
+        options.config_file = []
+    else:
+        # Make sure that explicitly provided files exist
+        if not os.path.exists(options.config_file):
+            parser.error("Config file '{0}' doesn't exist!".format(
+                options.config_file))
+        options.config_file = [options.config_file]
+
+    if not check:
+        return options, args
 
     if not options.list and not options.python_virtualenv:
         if not args:
-            print('You must provide a DEST_DIR or '
-                  'use current python virtualenv')
-            parser.print_help()
-            sys.exit(2)
+            parser.error('You must provide a DEST_DIR or '
+                         'use current python virtualenv')
 
         if len(args) > 1:
-            print('There must be only one argument: DEST_DIR (you gave %s)' % (
-                ' '.join(args)))
-            parser.print_help()
-            sys.exit(2)
+            parser.error('There must be only one argument: DEST_DIR '
+                         '(you gave: {0})'.format(' '.join(args)))
 
     return options, args
 
@@ -367,7 +405,7 @@ def callit(cmd, show_stdout=True, in_shell=False,
         line = stdout.readline()
         if not line:
             break
-        line = line.rstrip()
+        line = line.decode('utf-8').rstrip()
         all_output.append(line)
         if show_stdout:
             logger.info(line)
@@ -464,8 +502,10 @@ def build_node_from_src(env_dir, src_dir, node_src_dir, opt):
         # therefore we need to temporarily point python exec to the
         # python 2.* version in this case.
         try:
-            _, which_python2_output = callit(['which', 'python2'])
-            python2_path = which_python2_output[0].decode('utf-8')
+            _, which_python2_output = callit(
+                ['which', 'python2'], opt.verbose, True, node_src_dir, env
+            )
+            python2_path = which_python2_output[0]
         except (OSError, IndexError):
             raise OSError(
                 'Python >=3.0 virtualenv detected, but no python2 '
@@ -667,7 +707,7 @@ def print_node_versions():
         if not row:
             logger.info('\t'.join(rowx))
             break
-        rowx.append(row.replace('\n', ''))
+        rowx.append(row.decode('utf-8').replace('\n', ''))
         if pos % 8 == 0:
             logger.info('\t'.join(rowx))
             rowx = []
@@ -691,6 +731,8 @@ def get_env_dir(opt, args):
     if opt.python_virtualenv:
         if hasattr(sys, 'real_prefix'):
             return sys.prefix
+        elif hasattr(sys, 'base_prefix') and sys.base_prefix != sys.prefix:
+            return sys.prefix
         logger.error('No python virtualenv is available')
         sys.exit(2)
     else:
@@ -706,7 +748,9 @@ def main():
         Config._dump()
         return
 
-    Config._load()
+    opt, args = parse_args(check=False)
+    Config._load(opt.config_file, opt.verbose)
+
     opt, args = parse_args()
     if not opt.node or opt.node.lower() == "latest":
         opt.node = get_last_stable_node_version()
@@ -794,7 +838,8 @@ freeze () {
             npmls="npm ls"
             shift
         fi
-        NPM_LIST=$(eval ${npmls} | grep -E '^.{4}\w{1}'| grep -o -E "$re"| grep -v npm)
+        NPM_LIST=$(eval ${npmls} | grep -E '^.{4}\w{1}'| \
+                                   grep -o -E "$re"| grep -v npm)
     fi
 
     if [ -z "$@" ]; then
